@@ -11,10 +11,16 @@ use Moo;
 use IPC::Run3; # For run3().
 use Types::Standard qw/Any ArrayRef HasMethods HashRef Int Str/;
 
-our $VERSION = '2.59';
+our $VERSION = '2.60';
 
 my $DATA_SECTION = get_data_section; # load once
 my $DEFAULT_COMBINE = 1; # default for combine_node_and_port
+my %CONTEXT_QUOTING = (
+	label => '\\{\\}\\|<>\\s"',
+	label_legacy => '"',
+);
+my %PORT_QUOTING = map +($_ => sprintf "%%%02x", ord $_), qw(% \\ : " { } | < >);
+my $PORT_QUOTE_CHARS = join '', '[', (map quotemeta, sort keys %PORT_QUOTING), ']';
 
 has command =>
 (
@@ -269,8 +275,8 @@ sub add_edge
 		my $port = '';
 		if ($self->global->{combine_node_and_port}) {
 			($name, $port) = $self->_edge_name_port($name);
-		} elsif (exists $arg{$argname}) {
-			$port = ':' . delete $arg{$argname};
+		} elsif (defined(my $value = delete $arg{$argname})) {
+			$port = join ':', '', map qq{"$_"}, map escape_port($_), ref $value ? @$value : $value;
 		}
 		push @nodes, [ $name, $port ];
 		next if (my $nh = $self->node_hash)->{$name};
@@ -312,15 +318,15 @@ sub _compile_record {
 		$text = "{$text}" if $add_braces;
 	} elsif (ref $item eq 'HASH') {
 		my $port = $item->{port} || 0;
-		$text = escape_some_chars($item->{text} // '', $quote_more);
+		$text = escape_some_chars($item->{text} // '', $CONTEXT_QUOTING{$quote_more ? 'label' : 'label_legacy'});
 		if ($port) {
-			$port =~ s/^\s*<?/</;
-			$port =~ s/>?\s*$/>/;
-			$text = $text;
-			$text = "$port $text";
+			$port =~ s/^\s*<?//;
+			$port =~ s/>?\s*$//;
+			$port = escape_port($port);
+			$text = "<$port> $text";
 		}
 	} else {
-		$text = "<port".++$port_count."> " . escape_some_chars($item, $quote_more);
+		$text = "<port".++$port_count."> " . escape_some_chars($item, $CONTEXT_QUOTING{$quote_more ? 'label' : 'label_legacy'});
 	}
 	($port_count, $text);
 }
@@ -343,7 +349,7 @@ sub add_node {
 	} elsif ($arg{shape} && ( ($arg{shape} =~ /M?record/) || ( ($arg{shape} =~ /(?:none|plaintext)/) && ($label =~ /^</) ) ) ) {
 		# Do not escape anything.
 	} elsif ($label) {
-		$arg{label} = escape_some_chars($arg{label});
+		$arg{label} = escape_some_chars($arg{label}, $CONTEXT_QUOTING{label_legacy});
 	}
 	my $dot = $self->stringify_attributes(qq|"$name"|, \%arg);
 	push @{ $self->command }, _indent($dot, $self->scope);
@@ -421,8 +427,14 @@ sub default_subgraph
 
 } # End of default_subgraph.
 
+sub escape_port {
+	my ($s) = @_;
+	$s =~ s/($PORT_QUOTE_CHARS)/$PORT_QUOTING{$1}/g;
+	$s;
+}
+
 sub escape_some_chars {
-	my ($s, $quote_more) = @_;
+	my ($s, $quote_chars) = @_;
 	my @s        = split(//, $s);
 	my $label    = '';
 	for my $i (0 .. $#s) {
@@ -433,7 +445,7 @@ sub escape_some_chars {
 			if (substr($s, 0, 1) ne '<') {
 				$maybe = 1; # It's not a HTML label
 			}
-		} elsif ($quote_more and $s[$i] =~ /[\{\}\|<>"]/) {
+		} elsif ($quote_chars and $s[$i] =~ /[$quote_chars]/) {
 			$maybe = 1;
 		}
 		# Escape if not escaped.
@@ -615,7 +627,10 @@ sub from_graph {
 	my $global = { directed => $g->is_directed, %{delete $g_attrs{global}||{}} };
 	my $groups = delete $g_attrs{groups} || [];
 	if (ref $self) {
-		$self->$_($g_attrs{$_}) for sort keys %g_attrs;
+		for (sort keys %g_attrs) {
+			my $method = "default_$_";
+			$self->$method(%{ $g_attrs{$_} });
+		}
 	} else {
 		$self = $self->new(global => $global, %g_attrs);
 	}
@@ -778,8 +793,10 @@ treatment of double-colons).
 
 When the option is false, any name may be given to nodes, and edges can
 be created between them. To specify ports, give the additional parameter
-of C<tailport> or C<headport>. Also, C<add_node>'s treatment of labels
-is more DWIM, with C<{> etc being transparently quoted.
+of C<tailport> or C<headport>. To specify a compass point in addition,
+give array-refs with two values for these parameters. Also, C<add_node>'s
+treatment of labels is more DWIM, with C<{> etc being transparently
+quoted.
 
 =head4 directed => $Boolean
 
@@ -1225,11 +1242,6 @@ Add a edge from 1 node to another.
 
 $from_node_name and $to_node_name default to ''.
 
-If either of these node names is unknown, add_node(name => $node_name) is called automatically. The lack of
-attributes in this call means such nodes are created with the default set of attributes, and that may not
-be what you want. To avoid this, you have to call add_node(...) yourself, with the appropriate attributes,
-before calling add_edge(...).
-
 %hash is any edge attributes accepted as
 L<Graphviz attributes|https://www.graphviz.org/doc/info/attrs.html>.
 These are validated in exactly the same way as the edge parameters in the calls to
