@@ -30,7 +30,7 @@ has graph => (
     required => 0,
 );
 sub _build_graph {
-    GraphViz2->new(%GRAPHVIZ_ARGS)->from_graph(graphvizify($_[0]->as_graph));
+    GraphViz2->new(%GRAPHVIZ_ARGS)->from_graph(graphvizify($_[0]->as_graph, $_[0]->mode));
 }
 
 has stt => (
@@ -38,12 +38,18 @@ has stt => (
     required => 0,
 );
 
+has mode => (
+    is       => 'rw',
+    required => 0,
+);
+
 sub _quote { my $t = $_[0]; $t =~ s/\\/\\\\/g; $t; }
 
 sub create {
     my ($self, %arg) = @_;
     $self->stt($arg{stt});
-    $self->graph->from_graph(graphvizify($self->as_graph));
+    $self->mode($arg{mode}) if exists $arg{mode};
+    $self->graph->from_graph(graphvizify($self->as_graph, $self->mode));
     return $self;
 }
 
@@ -53,22 +59,77 @@ sub to_graph {
     for my $line (split /\n/, $stt) {
         $line =~ s/^\s*\[?//;
         $line =~ s/\s*(],?)?$//;
-        # The first 2 '\'s are just to fix the syntax highlighting in UltraEdit.
         my ($f, $re, $t) = quotewords('\s*,\s*', 0, $line);
-        $g->set_edge_attribute($f, $t, re => $re);
+        $g->set_edge_attributes($f, $t, { type => 'transition', re => $re });
+        $g->set_vertex_attribute($_, type => 'state') for $f, $t;
     }
     return $g;
 }
 
 sub graphvizify {
-    my ($g) = @_;
-    for my $v ($g->vertices) {
-        $g->set_edge_attribute(
-            @$_,
-            graphviz => { label => "/" . _quote($g->get_edge_attribute(@$_, 're')) . "/" },
-        ) for $g->edges_from($v);
+    my ($g, $mode) = @_;
+    $mode //= 're_structs';
+    if ($mode eq 're_nodes') {
+        my %state2re_s;
+        for my $v (grep $g->get_vertex_attribute($_, 'type') eq 'state', $g->vertices) {
+            for my $e (grep $g->get_edge_attribute(@$_, 'type') eq 'transition', $g->edges_from($v)) {
+                my ($re, $t) = ($g->get_edge_attribute(@$e, 're'), $e->[1]);
+                $state2re_s{$t}{ my $re_node_id = "$re:$t" } = 1;
+                $g->set_vertex_attributes($re_node_id, {
+                    type => 're',
+                    graphviz => {
+                        label => "/" . _quote($re) . "/",
+                        color => 'black', shape => 'box',
+                    },
+                });
+                $g->set_edge_attribute($v, $re_node_id, type => 'state_re');
+                $g->set_edge_attribute($re_node_id, $t, type => 're_state');
+                $g->delete_edge(@$e);
+            }
+        }
+        my @groups = map +{
+            attributes => { name => "cluster_$_", subgraph => { rank => "TB" } },
+            nodes => [ $_, sort keys %{$state2re_s{$_}} ],
+        }, sort keys %state2re_s;
+        $g->set_graph_attribute(
+            graphviz => {
+                global => $GRAPHVIZ_ARGS{global},
+                graph => { clusterrank => 'local', compound => 1 },
+                groups => \@groups,
+            },
+        );
+    } elsif ($mode eq 're_structs') {
+        for my $v (grep $g->get_vertex_attribute($_, 'type') eq 'state', $g->vertices) {
+            my @ins = grep $_->[1] eq 'transition',
+                map [ $_->[0], @{ $g->get_edge_attributes(@$_) }{qw(type re)} ],
+                $g->edges_to($v);
+            $g->set_edge_attribute(
+                $_->[0], $v,
+                graphviz => { tailport => 'state', headport => [ $_->[2], 'w' ] },
+            ) for @ins;
+            my %unique_re = map +($_->[2] => undef), @ins;
+            $g->set_vertex_attribute($v, graphviz => {
+                shape => 'record',
+                label => [
+                    [ map +{ text => "/"._quote($_)."/", port => $_ }, sort keys %unique_re ],
+                    { text => _quote($v), port => 'state' },
+                ],
+            });
+        }
+        $g->set_graph_attribute(graphviz => { global => $GRAPHVIZ_ARGS{global} });
+    } elsif ($mode eq 're_edges') {
+        for my $v (grep $g->get_vertex_attribute($_, 'type') eq 'state', $g->vertices) {
+            for my $e (grep $g->get_edge_attribute(@$_, 'type') eq 'transition', $g->edges_from($v)) {
+                $g->set_edge_attribute(
+                    @$e,
+                    graphviz => { label => "/" . _quote($g->get_edge_attribute(@$e, 're')) . "/" },
+                );
+            }
+        }
+        $g->set_graph_attribute(graphviz => { global => $GRAPHVIZ_ARGS{global} });
+    } else {
+        die "Unknown STT visualisation mode '$mode'";
     }
-    $g->set_graph_attribute(graphviz => { global => $GRAPHVIZ_ARGS{global} });
     $g;
 }
 
@@ -91,8 +152,11 @@ L<GraphViz2::Parse::STT> - Visualize a Set::FA::Element state transition table a
     # populate a GraphViz2 object with a Graph::Directed of a parser
     my $gv = GraphViz2->from_graph(GraphViz2::Parse::STT::graphvizify($gd));
 
+    # visualise with another mode
+    $gd = GraphViz2::Parse::STT::graphvizify($gd, 're_nodes'); # or 're_edges'
+
     # OO interface, using lazy-built attributes
-    my $gvp = GraphViz2::Parse::STT->new(stt => $stt);
+    my $gvp = GraphViz2::Parse::STT->new(stt => $stt, mode => 're_structs');
     my $gd = $gvp->as_graph; # Graph::Directed object
     # or supply a suitable Graph::Directed object
     my $gvp = GraphViz2::Parse::STT->new(as_graph => $gd);
@@ -134,17 +198,25 @@ This is the recommended interface.
 Given STT text, returns a L<Graph::Directed> object describing the finite
 state machine for it.
 
+The nodes are all states, and the edges are regular expressions that
+cause a transition to another state.
+
 =head2 graphvizify
 
-    my $gv = GraphViz2->from_graph(GraphViz2::Parse::STT::graphvizify($gd));
+    my $gv = GraphViz2->from_graph(GraphViz2::Parse::STT::graphvizify($gd, $mode));
 
 Mutates the given graph object to add to it the C<graphviz> attributes
 visualisation "hints" that will make the L<GraphViz2/from_graph> method
 visualise this regular expression in the most meaningful way, including
 labels and groupings.
 
-It is idempotent as it simply sets the C<graphviz> attribute of the
-relevant graph entities.
+It is idempotent, but in C<re_nodes> mode, it deletes the transition
+edges and replaces them with additional nodes and edges.
+
+If a second argument is given, it will be the visualisation "mode". The
+default is C<re_structs>. Also available is C<re_nodes>, and C<re_edges>
+where the regular expressions are simply added as labels to the
+state-transition edges.
 
 Returns the graph object for convenience.
 
@@ -203,7 +275,11 @@ L</as_graph>.
             node   => {color => 'blue', shape => 'oval'},
     );
 
-=head2 create(regexp => $regexp)
+=head3 mode
+
+The mode to be used by L</graphvizify>.
+
+=head2 create(regexp => $regexp, mode => $mode)
 
 DEPRECATED. Mutates the object to set the C<stt> attribute, then
 accesses the C<as_graph> attribute (possibly lazy-building it), then
